/*
 * Definitions for PALs/GALs
 */

/* PAL pin types */
#define GLOBAL_CLK	1	/* Global clock for registered outputs */
#define GLOBAL_OE	2	/* global enable for registered outputs */
#define INPUT_PIN	4	/* can be used as input or feedback */
#define OUTPUT_PIN	8	/* can be used as output */
#define IO_PIN		(INPUT_PIN|OUTPUT_PIN)
#define VCC		0x10
#define GND		0x20
#define GLOBAL_RESET	0x40	/* global reset pin */
#define DED_INPUT	0x80	/* dedicated (fast) input pin */

/* output pin types */
#define POL_HIGH	0x100
#define POL_LOW		0x200
#define REG		0x400
#define COMB		0x800
#define SEP_OE		0x1000	/* separate row for output enable */

/* input/feedback pin characteristics */
#define FB_INVERTED	0x2000	/* internal feedback path contains inversion */
#define FB_FROM_REG	0x4000	/* reg output has feedback from register */
#define FB_FROM_OR	0x8000	/* comb output has feedback from OR gate */

#define V8MACRO		(POL_HIGH|POL_LOW|REG|COMB)
#define V8MAC_COMB	(POL_HIGH|POL_LOW|COMB)
#define V10MACRO	(V8MACRO|SEP_OE|FB_FROM_REG|FB_INVERTED)

struct pin {
	long	pin_type;	/* contains bits defined above */
	short	in_col;		/* column pair # for input or feedback */
	short	out_frow;	/* first row number for output */
	short	out_nrow;	/* # rows for output */
	short	out_idx;	/* index into polarity/architecture arrays */
};

#define MAX_PIN		24

struct pal {
	short	num_pins;	/* total # pins */
	short	num_outputs;	/* total # outputs */
	short	num_cols;	/* # columns in matrix */
	short	num_rows;	/* # rows in matrix */
	short	gbl_clk;	/* global clock pin # for registered outputs */
	short	gbl_oe;		/* global output enable */
	struct pin *pins;	/* info about each pin */
	void	(*sp_init)();	/* function called before processing pins */
	void	(*sp_pdecl)();	/* function for processing pin decl */
	void	(*sp_pal)();	/* function called at end of decl */
	void	(*sp_dev)();	/* function for device-specific stuff */
	int	sp_arg;		/* param for sp_dev */
};

struct pal_tbl {
	char *pal_name;
	struct pal *pal_defn;
};

extern struct pal_tbl pal_table[];

struct pin_ass {
	short		ass_attr;	/* specified attrs for this pin */
	struct expr	*ass_sig;	/* assigned signal */
};

